@@ -98,7 +98,7 @@ from contextlib import closing
 import _udev
 
 
-__version__ = '0.2'
+__version__ = '0.3'
 __all__ = ['Context', 'Device']
 
 
@@ -195,12 +195,18 @@ class Enumerator(object):
 
     Before iteration the device list can be filtered by subsystem or by
     property values using :meth:`match_subsystem` and
-    :meth:`match_property`.  All added filters must match for a device to be
-    included in the device list, when eventually iterating over this object.
-    The filter methods return the instance itself again, thus supporting
-    method chaining to apply many filters in a row.
+    :meth:`match_property`.  Multiple subsystem (property) filters are
+    combined using a logical OR, filters of different types are combined
+    using a logical AND.  The following filter for instance::
 
-    Once a filter was added, it cannot be removed.  Create a new object
+        devices.match_subsystem('block').match_property(
+            'ID_TYPE', 'disk').match_property('DEVTYPE', 'disk')
+
+    means the following::
+
+        subsystem == 'block' and (ID_TYPE == 'disk' or DEVTYPE == 'disk')
+
+    Once added, a filter cannot be removed anymore.  Create a new object
     instead.
     """
 
@@ -216,6 +222,7 @@ class Enumerator(object):
             raise TypeError('Invalid context object')
         self.context = context
         self._enumerator = libudev.udev_enumerate_new(context._context)
+        self._parents = []
 
     def __del__(self):
         libudev.udev_enumerate_unref(self._enumerator)
@@ -253,6 +260,16 @@ class Enumerator(object):
                     _property_value_to_bytes(value))
         return self
 
+    def match_children(self, device):
+        """
+        Include all *direct* children of the given ``device``.  A child is a
+        device, whose :attr:`Device.parent` points to ``device``.
+
+        Return the instance again.
+        """
+        self._parents.append(device)
+        return self
+
     def __iter__(self):
         """
         Iterate over all matching devices.
@@ -262,7 +279,10 @@ class Enumerator(object):
         _check_call(libudev.udev_enumerate_scan_devices, self._enumerator)
         entry = libudev.udev_enumerate_get_list_entry(self._enumerator)
         for name in _udev_list_iterate(entry):
-            yield Device.from_sys_path(self.context, name)
+            device = Device.from_sys_path(self.context, name)
+            if (not self._parents) or any(device.parent == p for p
+                                          in self._parents):
+                yield device
 
 
 class Device(Mapping):
@@ -278,9 +298,20 @@ class Device(Mapping):
     Aside of the properties, a device also has a set of udev-specific
     attributes like the path inside ``sysfs``.
 
-    Devices can compare equal and unequal to other devices and to strings
-    (based on :attr:`device_path`).  Moreover devices are hashable.  Therefore
-    :class:`Device` objects can be used as keys in dictionaries and sets.
+    :class:`Device` objects compare equal and unequal to other devices and
+    to strings (based on :attr:`device_path`).  However, there is no
+    ordering on :class:`Device` objects, and the corresponding operators
+    ``>``, ``<``, ``<=`` and ``>=`` raise :exc:`~exceptions.TypeError`.
+
+    .. warning::
+
+       Do **never** use object identity (``is`` operator) to compare
+       :class:`Device` objects.  :mod:`udev` may create multiple
+       :class:`Device` objects for the same device.  Instead simply compare
+       devices by value using ``==`` or ``!=``.
+
+    :class:`Device` objects are hashable and can therefore be used as keys
+    in dictionaries and sets.
     """
 
     @classmethod
@@ -323,6 +354,23 @@ class Device(Mapping):
         # reference
         return Device(self.context, libudev.udev_device_ref(parent))
 
+    @property
+    def children(self):
+        """
+        Yield all direct children of this device.
+
+        .. note::
+
+           As the underlying library does not provide any means to directly
+           query the children of a device, this property performs a linear
+           search through all devices.
+
+        Return an iterable yielding a :class:`Device` object for each direct
+        child of this device.
+        """
+        for device in self.context.list_devices().match_children(self):
+                yield device
+
     def traverse(self):
         """
         Traverse all parent devices of this device from bottom to top.
@@ -477,6 +525,18 @@ class Device(Mapping):
         else:
             return self.device_path != other
 
+    def __gt__(self, other):
+        raise TypeError('Device not orderable')
+
+    def __lt__(self, other):
+        raise TypeError('Device not orderable')
+
+    def __le__(self, other):
+        raise TypeError('Device not orderable')
+
+    def __ge__(self, other):
+        raise TypeError('Device not orderable')
+
 
 class Monitor(object):
     """
@@ -524,13 +584,14 @@ class Monitor(object):
         ``context`` is the :class:`Context` to use.  ``source`` is the event
         source.  As of now, two sources are available:
 
-        - ``'udev'`` (the default):  Events emitted after udev as registered
-          and configured the device.  This is the absolutely recommended
-          source for applications.
-        - ``'kernel'``:  Events emitted directly after the kernel has seen
-          the device.  The device has not yet been configured by udev and
-          might not be usable at all.  **Never** use this, unless you know
-          what you are doing.
+        ``'udev'`` (the default)
+          Events emitted after udev as registered and configured the device.
+          This is the absolutely recommended source for applications.
+
+        ``'kernel'``
+          Events emitted directly after the kernel has seen the device.  The
+          device has not yet been configured by udev and might not be usable
+          at all.  **Never** use this, unless you know what you are doing.
 
         Return a new :class:`Monitor` object, which is connected to the
         given source.  Raise :exc:`~exceptions.ValueError`, if an invalid
@@ -639,12 +700,23 @@ class Monitor(object):
         Receive a single device from the monitor.
 
         The caller must make sure, that there are events available in the
-        event queue.  The call may block, until a device is available.
+        event queue.  The call blocks, until a device is available.
 
-        If a device was available, return ``(action, device)``.  ``action``
-        is a string describing the action (e.g. ``'add'``, ``'remove'``).
-        ``device`` is :class:`Device` object describing the device.  If no
-        device was available, return ``None``.
+        If a device was available, return ``(action, device)``.  ``device``
+        is :class:`Device` object describing the device.  ``action`` is a
+        string describing the action.  udev informs about the following
+        actions:
+
+        ``'add'``
+          A device has been added (e.g. a USB device was plugged in)
+        ``'remove'``
+          A device has been removed (e.g. a USB device was unplugged)
+        ``'change'``
+          Something about the device changed (e.g. a device property)
+        ``'move'``
+          The device was renamed, moved, or re-parented
+
+        If no device was available, return ``None``.
         """
         device_p = libudev.udev_monitor_receive_device(self._monitor)
         if not device_p:
